==> kameleoon_client_flutter-2.0.0.tar.gz: lib/src/internal/kameleoon_client_impl_web.dart <==
@JS()
library kameleoon_client.js;

import 'dart:async';
import "dart:convert";
import 'dart:js_util';

import 'package:flutter/services.dart';
import 'package:js/js.dart';
import 'package:js/js_util.dart';

import '../kameleoon_client.dart';
import '../kameleoon_configuration.dart';
import '../kameleoon_data.dart';
import '../kameleoon_exception.dart';
import 'kameleoon_client_callback.dart';

/// Implementation of [KameleoonClient] for Web platform
class KameleoonClientImpl extends KameleoonClient {
  late final _KameleoonClientJs _kameleoonClientJs;

  bool ready = false;
  List<KameleoonClientCallback> _callbacks = [];

  KameleoonClientImpl(String siteCode, KameleoonConfiguration config, MethodChannel channel) {
    _kameleoonClientJs = _KameleoonClientJs(siteCode,
      _SDKConfigurationTypeJs(updateInterval: config.refreshIntervalMinutes, environment: config.environment));
  }

  ////////////////////////////////
  /// PUBLIC METHODS
  ////////////////////////////////

  bool? isReady() {
    return this.ready;
  }

  void runWhenReady(Function readyCallback, Function failCallback, [Duration? timeout]) async {
    if (this.ready) {
      readyCallback();
    } else {
      final runWhenReadyTimeout = timeout ?? Duration(seconds: 2);
      final callback = KameleoonClientCallback(readyCallback, failCallback);
      _callbacks.add(callback);
      promiseToFuture(_kameleoonClientJs.initialize())
        .timeout(runWhenReadyTimeout)
        .then((value) {
          this.ready = true;
          _callbackAfterStatusChange(true, callback);
        });
      Future.delayed(runWhenReadyTimeout, () => _callbackAfterStatusChange(false, callback));
    }
  }

  Future<void> trackConversion(String visitorCode, int goalId, [double? revenue = 0]) {
    return _run(() =>_kameleoonClientJs.trackConversion(
      _TrackConversionParamsType(visitorCode: visitorCode, goalId: goalId, revenue: revenue ?? 0)));
  }

  Future<void> addData(String visitorCode, List<Data> listData) async {
    return _run(() {
      final listDataJS = listData.map((data) {
        switch (data.dataType) {
          case Data.TypeCustomData:
            final customData = data as CustomData;
            final customDataJs = _CustomDataJs(customData.id, []);
            customDataJs.value = customData.values;
            return customDataJs;
          case Data.TypePageView:
            final pageView = data as PageView;
            return _PageViewJs(
              _PageViewParametersType(
                urlAddress: pageView.url,
                title: pageView.title,
                referrers: pageView.referrers)
              );
          case Data.TypeConversion:
            final conversion = data as Conversion;
            return _ConversionJs(
              _ConversionParametersType(
                goalId: conversion.goalId,
                revenue: conversion.revenue,
                negative: conversion.negative)
            );
          case Data.TypeBrowser:
            final browser = data as Browser;
            return _BrowserJs(browser.browser);
          case Data.TypeDevice:
            final device = data as Device;
            return _DeviceJs(device.device);
          default:
            print("Unsupported type: ${data.dataType} wasn't added");
            return CustomData(-1, "");
        }
      });
      return callMethod(_kameleoonClientJs, "addData", [visitorCode, ...listDataJS]);
    });
  }

  Future<void> flush([String? visitorCode]) async {
    return _run(() {
      if (visitorCode != null) {
        return _kameleoonClientJs.flushData(visitorCode);
      } else {
        print("Flushing for all users isn't supported. Please specify the visitorCode explicitly");
      }
      return Future.value();
    });
  }

  Future<int> triggerExperiment(String visitorCode, int experimentId) {
    return _run(() => _kameleoonClientJs.triggerExperiment(visitorCode, experimentId));
  }

  Future<bool> isFeatureActive(String visitorCode, String featureKey) {
    return _run(() => _kameleoonClientJs.isFeatureFlagActive(visitorCode, featureKey));
  }

  Future<String> getFeatureVariationKey(String visitorCode, String featureKey) {
    return _run(() => _kameleoonClientJs.getFeatureFlagVariationKey(visitorCode, featureKey));
  }

  Future<Object?> getFeatureVariable(String visitorCode, String featureKey, String variableKey) async {
    return _jsToDart(await _run(() => _kameleoonClientJs.getFeatureFlagVariable(
      _GetFeatureFlagVariableParamsType(visitorCode: visitorCode, featureKey: featureKey, variableKey: variableKey))
    ))['value'];
  }

  Future<dynamic> getVariationAssociatedData(int variationId) async {
    return _jsToDart(await _run(() => _kameleoonClientJs.getExperimentVariationData(variationId)));
  }

  Future<dynamic> obtainVariationAssociatedData(int variationId) => getVariationAssociatedData(variationId);


  Future<Map<String, Object>> getFeatureAllVariables(String featureKey, String variationKey) {
    print("This method isn't implemented for web yet, and will return empty map");
    return Future.value(Map<String, Object>());
  }

  Future<Object?> obtainFeatureVariable(var featureKeyOrID, String variableKey) async {
    // "off" == default variation which always exists
    return getFeatureAllVariables(featureKeyOrID.toString(), "off").then((value) {
      if (value.containsKey(variableKey)) {
        return value[variableKey];
      } else {
        throw VariableNotFound(variableKey: variableKey);
      }
    });
  }

  Future<dynamic> getRemoteData(String key) async {
    return _jsToDart(await promiseToFuture(_kameleoonClientJs.getRemoteData(key)));
  }

  Future<dynamic> retrieveDataFromRemoteSource(String key) async => getRemoteData(key);

  @override
  void updateConfigurationHandler(Function? handler) {
    if (handler != null) {
      _kameleoonClientJs.onConfigurationUpdate(allowInterop(handler));
    } else {
      _kameleoonClientJs.onConfigurationUpdate(allowInterop(() {}));
    }
  }

  @override
  Future<List<int>> getExperimentList([String visitorCode= "", bool onlyAllocated = true]) {
    List<int> result;
    if (visitorCode.isEmpty) {
      result = _kameleoonClientJs.getExperiments().map((e) => e.id).toList();
    } else {
      result = _kameleoonClientJs.getVisitorExperiments(visitorCode, onlyAllocated).map((e) => e.id).toList();
    }
    return Future.value(result);
  }

  @override
  Future<List<String>> getFeatureList() {
    return Future.value(_kameleoonClientJs.getFeatureFlags().map((e) => e.key).toList());
  }

  @override
  Future<List<String>> getFeatureListActive(String visitorCode) {
    return Future.value(_kameleoonClientJs.getVisitorFeatureFlags(visitorCode).map((e) => e.key).toList());
  }

  Future<String> getVisitorCode({String? domain, String? defaultVisitorCode}) =>
    _KameleoonUtilsJs.getVisitorCode(domain ?? "", defaultVisitorCode);

  Future<String?> obtainVisitorCode([String? defaultVisitorCode]) =>
    getVisitorCode(defaultVisitorCode: defaultVisitorCode);

  ////////////////////////////////
  /// PRIVATE METHODS / HELPERS
  ////////////////////////////////
  ///

  // forget for web does nothing but it's required on mobile platforms
  void forget() {}

  // nativeMethodCallHandler for web does nothing but it's required on mobile platforms
  void nativeMethodCallHandler(MethodCall call) {}

  void _callbackAfterStatusChange(bool isReady, [KameleoonClientCallback? callback]) {
    List<int> idsToDelete = [];
    for (int i = 0; i < _callbacks.length; i++) {
      if (isReady) {
        _callbacks[i].readyCallback();
        idsToDelete.add(i);
      } else {
        if (_callbacks[i] == callback) {
          callback?.failCallback?.call();
          idsToDelete.add(i);
        }
      }
    }

    for (int i = idsToDelete.length - 1; i >= 0; i--) {
      _callbacks.removeAt(i);
    }
  }

  Future<T> _run<T>(Future<T> Function() run) async {
    try {
      return await run();
    } catch(error) {
      final exception = _jsToDart(error);
      final message = error.toString();
      switch (exception["errorType"]) {
        case _Constants.Initialization:
          throw SDKNotReady(message);
        case _Constants.ExperimentConfigurationNotFound:
          throw ExperimentNotFound(message);
        case _Constants.FeatureFlagConfigurationNotFound:
          throw FeatureNotFound(message);
        case _Constants.FeatureFlagVariableNotFound:
          throw VariableNotFound(message: message);
        case _Constants.NotTargeted:
          throw NotTargeted(message);
        case _Constants.NotAllocated:
          throw NotAllocated(message);
        case _Constants.VisitorCodeMaxLength:
        case _Constants.VisitorCodeEmpty:
          throw VisitorCodeNotValid(message);
        case _Constants.ClientConfiguration:
          throw CredentialsNotFound(message);
        case _Constants.VariationNotFound:
          throw VariationNotFound(message);
        default:
          throw Exception(exception);
      }
    }
  }
}

class _Constants {
  static const ExperimentConfigurationNotFound = 'ExperimentConfigurationNotFound',
  VariationNotFound = 'VariationNotFound',
  FeatureFlagConfigurationNotFound = 'FeatureFlagConfigurationNotFound',
  FeatureFlagVariableNotFound = 'FeatureFlagVariableNotFound',
  NotTargeted = 'NotTargeted',
  NotAllocated = 'NotAllocated',
  VisitorCodeMaxLength = 'VisitorCodeMaxLength',
  VisitorCodeEmpty = 'VisitorCodeEmpty',
  ClientConfiguration = 'ClientConfiguration',
  Initialization = 'Initialization';
}

/// Helper class KameleoonUtilsJs to make interaction with JS sdk (web platform)
@JS("kameleoon.KameleoonUtils")
class _KameleoonUtilsJs {
  external static Future<String> getVisitorCode(String domain, String? defaultVisitorCode);
}

/// Helper class KameleoonClientJs to make interaction with JS sdk (web platform)
@JS("kameleoon.KameleoonClient")
class _KameleoonClientJs {
  external _KameleoonClientJs(String siteCode, _SDKConfigurationTypeJs configuration);
  external initialize();
  external runWhenReady(Function readyCallback, Function failCallback,
      [int timeout]);
  external Future<void> trackConversion(_TrackConversionParamsType object);
  external Future<void> addData(String visitorCode, Iterable<_KameleoonDataTypeJs> args);
  external Future<dynamic> obtainFeatureVariable(var featureKeyOrId, String variableKey);
  external Future<dynamic> getFeatureFlagVariable(_GetFeatureFlagVariableParamsType object);
  external Future<String> getFeatureFlagVariationKey(String visitorCode, String featureKey);
  external Future<dynamic> getExperimentVariationData(int variationId);
  external Future<void> flushData(String visitorCode);
  external Future<bool> isFeatureFlagActive(String visitorCode, String featureKey);
  external Future<int> triggerExperiment(String visitorCode, int experimentID);
  external Future<dynamic> getRemoteData(String key);
  external void onConfigurationUpdate(Function func);
  external List<_ExperimentType> getExperiments();
  external List<_ExperimentType> getVisitorExperiments(String visitorCode, bool? isAllocated);
  external List<_FeatureFlagType> getFeatureFlags();
  external List<_FeatureFlagType> getVisitorFeatureFlags(String visitorCode);

}

@JS("kameleoon.KameleoonDataType")
class _KameleoonDataTypeJs {
  external _KameleoonDataTypeJs();
}

@JS("kameleoon.CustomData")
class _CustomDataJs extends _KameleoonDataTypeJs {
  external Iterable<String> value;
  external _CustomDataJs(int id, Iterable<String> value);
}

@JS("kameleoon.Browser")
class _BrowserJs extends _KameleoonDataTypeJs {
  external _BrowserJs(int id);
}

@JS("kameleoon.Device")
class _DeviceJs extends _KameleoonDataTypeJs {
  external _DeviceJs(String device);
}

@JS("kameleoon.PageView")
class _PageViewJs extends _KameleoonDataTypeJs {
  external _PageViewJs(_PageViewParametersType object);
}

@JS("kameleoon.Conversion")
class _ConversionJs extends _KameleoonDataTypeJs {
  external _ConversionJs(_ConversionParametersType object);
}

@JS()
@anonymous
class _SDKConfigurationTypeJs {
  external factory _SDKConfigurationTypeJs({int? updateInterval, String? environment});
}

@JS()
@anonymous
class _GetFeatureFlagVariableParamsType {
  external factory _GetFeatureFlagVariableParamsType({String visitorCode, String featureKey, String variableKey});
}

@JS()
@anonymous
class _TrackConversionParamsType {
  external factory _TrackConversionParamsType({String visitorCode, int goalId, double revenue});
}

@JS()
@anonymous
class _ConversionParametersType {
  external factory _ConversionParametersType({int goalId, double revenue, bool negative});
}

@JS()
@anonymous
class _PageViewParametersType {
  external factory _PageViewParametersType({String urlAddress, String title, Iterable referrers});
}

@JS()
@anonymous
class _ExperimentType {
  external int id;
  external factory _ExperimentType();
}

@JS()
@anonymous
class _FeatureFlagType {
  external String key;
  external factory _FeatureFlagType();
}


Map _jsToDart(jsObject) {
  return jsonDecode(stringify(jsObject));
}

@JS('JSON.stringify')
external String stringify(Object obj);

==> kameleoon_client_flutter-2.0.2.tar.gz: lib/src/internal/kameleoon_client_impl_web.dart <==
@JS()
library kameleoon_client.js;

import 'dart:async';
import "dart:convert";
import 'dart:js_util';

import 'package:flutter/services.dart';
import 'package:js/js.dart';
import 'package:js/js_util.dart';

import '../kameleoon_client.dart';
import '../kameleoon_configuration.dart';
import '../kameleoon_data.dart';
import '../kameleoon_exception.dart';
import 'kameleoon_client_callback.dart';

/// Implementation of [KameleoonClient] for Web platform
class KameleoonClientImpl extends KameleoonClient {
  late final _KameleoonClientJs _kameleoonClientJs;

  bool ready = false;
  List<KameleoonClientCallback> _callbacks = [];

  KameleoonClientImpl(String siteCode, KameleoonConfiguration config, MethodChannel channel) {
    _kameleoonClientJs = _KameleoonClientJs(siteCode,
      _SDKConfigurationTypeJs(updateInterval: config.refreshIntervalMinutes, environment: config.environment));
  }

  ////////////////////////////////
  /// PUBLIC METHODS
  ////////////////////////////////

  bool? isReady() {
    return this.ready;
  }

  void runWhenReady(Function readyCallback, Function failCallback, [Duration? timeout]) async {
    if (this.ready) {
      readyCallback();
    } else {
      final runWhenReadyTimeout = timeout ?? Duration(seconds: 2);
      final callback = KameleoonClientCallback(readyCallback, failCallback);
      _callbacks.add(callback);
      promiseToFuture(_kameleoonClientJs.initialize())
        .timeout(runWhenReadyTimeout)
        .then((value) {
          this.ready = true;
          _callbackAfterStatusChange(true, callback);
        });
      Future.delayed(runWhenReadyTimeout, () => _callbackAfterStatusChange(false, callback));
    }
  }

  Future<void> trackConversion(String visitorCode, int goalId, [double? revenue = 0]) {
    return _run(() =>_kameleoonClientJs.trackConversion(
      _TrackConversionParamsType(visitorCode: visitorCode, goalId: goalId, revenue: revenue ?? 0)));
  }

  Future<void> addData(String visitorCode, List<Data> listData) async {
    return _run(() {
      final listDataJS = listData.map((data) {
        switch (data.dataType) {
          case Data.TypeCustomData:
            final customData = data as CustomData;
            final customDataJs = _CustomDataJs(customData.id, []);
            customDataJs.value = customData.values;
            return customDataJs;
          case Data.TypePageView:
            final pageView = data as PageView;
            return _PageViewJs(
              _PageViewParametersType(
                urlAddress: pageView.url,
                title: pageView.title,
                referrers: pageView.referrers)
              );
          case Data.TypeConversion:
            final conversion = data as Conversion;
            return _ConversionJs(
              _ConversionParametersType(
                goalId: conversion.goalId,
                revenue: conversion.revenue,
                negative: conversion.negative)
            );
          case Data.TypeBrowser:
            final browser = data as Browser;
            return _BrowserJs(browser.browser, browser.version);
          case Data.TypeDevice:
            final device = data as Device;
            return _DeviceJs(device.device);
          default:
            print("Unsupported type: ${data.dataType} wasn't added");
            return null;
        }
      }).where((x) => x != null);
      return callMethod(_kameleoonClientJs, "addData", [visitorCode, ...listDataJS]);
    });
  }

  Future<void> flush([String? visitorCode]) async {
    return _run(() {
      if (visitorCode != null) {
        return _kameleoonClientJs.flushData(visitorCode);
      } else {
        print("Flushing for all users isn't supported. Please specify the visitorCode explicitly");
      }
      return Future.value();
    });
  }

  Future<int> triggerExperiment(String visitorCode, int experimentId) {
    return _run(() => _kameleoonClientJs.triggerExperiment(visitorCode, experimentId));
  }

  Future<bool> isFeatureActive(String visitorCode, String featureKey) {
    return _run(() => _kameleoonClientJs.isFeatureFlagActive(visitorCode, featureKey));
  }

  Future<String> getFeatureVariationKey(String visitorCode, String featureKey) {
    return _run(() => _kameleoonClientJs.getFeatureFlagVariationKey(visitorCode, featureKey));
  }

  Future<Object?> getFeatureVariable(String visitorCode, String featureKey, String variableKey) async {
    return _jsToDart(await _run(() => _kameleoonClientJs.getFeatureFlagVariable(
      _GetFeatureFlagVariableParamsType(visitorCode: visitorCode, featureKey: featureKey, variableKey: variableKey))
    ))['value'];
  }

  Future<dynamic> getVariationAssociatedData(int variationId) async {
    return _jsToDart(await _run(() => _kameleoonClientJs.getExperimentVariationData(variationId)));
  }

  Future<dynamic> obtainVariationAssociatedData(int variationId) => getVariationAssociatedData(variationId);


  Future<Map<String, Object>> getFeatureAllVariables(String featureKey, String variationKey) {
    print("This method isn't implemented for web yet, and will return empty map");
    return Future.value(Map<String, Object>());
  }

  Future<Object?> obtainFeatureVariable(var featureKeyOrID, String variableKey) async {
    // "off" == default variation which always exists
    return getFeatureAllVariables(featureKeyOrID.toString(), "off").then((value) {
      if (value.containsKey(variableKey)) {
        return value[variableKey];
      } else {
        throw VariableNotFound(variableKey: variableKey);
      }
    });
  }

  Future<dynamic> getRemoteData(String key) async {
    return _jsToDart(await promiseToFuture(_kameleoonClientJs.getRemoteData(key)));
  }

  Future<dynamic> retrieveDataFromRemoteSource(String key) async => getRemoteData(key);

  @override
  void updateConfigurationHandler(Function? handler) {
    if (handler != null) {
      _kameleoonClientJs.onConfigurationUpdate(allowInterop(handler));
    } else {
      _kameleoonClientJs.onConfigurationUpdate(allowInterop(() {}));
    }
  }

  @override
  Future<List<int>> getExperimentList([String visitorCode= "", bool onlyAllocated = true]) {
    List<int> result;
    if (visitorCode.isEmpty) {
      result = _kameleoonClientJs.getExperiments().map((e) => e.id).toList();
    } else {
      result = _kameleoonClientJs.getVisitorExperiments(visitorCode, onlyAllocated).map((e) => e.id).toList();
    }
    return Future.value(result);
  }

  @override
  Future<List<String>> getFeatureList() {
    return Future.value(_kameleoonClientJs.getFeatureFlags().map((e) => e.key).toList());
  }

  @override
  Future<List<String>> getFeatureListActive(String visitorCode) {
    return Future.value(_kameleoonClientJs.getVisitorFeatureFlags(visitorCode).map((e) => e.key).toList());
  }

  Future<String> getVisitorCode({String? domain, String? defaultVisitorCode}) =>
    _KameleoonUtilsJs.getVisitorCode(domain ?? "", defaultVisitorCode);

  Future<String?> obtainVisitorCode([String? defaultVisitorCode]) =>
    getVisitorCode(defaultVisitorCode: defaultVisitorCode);

  ////////////////////////////////
  /// PRIVATE METHODS / HELPERS
  ////////////////////////////////
  ///

  // forget for web does nothing but it's required on mobile platforms
  void forget() {}

  // nativeMethodCallHandler for web does nothing but it's required on mobile platforms
  void nativeMethodCallHandler(MethodCall call) {}

  void _callbackAfterStatusChange(bool isReady, [KameleoonClientCallback? callback]) {
    List<int> idsToDelete = [];
    for (int i = 0; i < _callbacks.length; i++) {
      if (isReady) {
        _callbacks[i].readyCallback();
        idsToDelete.add(i);
      } else {
        if (_callbacks[i] == callback) {
          callback?.failCallback?.call();
          idsToDelete.add(i);
        }
      }
    }

    for (int i = idsToDelete.length - 1; i >= 0; i--) {
      _callbacks.removeAt(i);
    }
  }

  Future<T> _run<T>(Future<T> Function() run) async {
    try {
      return await run();
    } catch(error) {
      final exception = _jsToDart(error);
      final message = error.toString();
      switch (exception["errorType"]) {
        case _Constants.Initialization:
          throw SDKNotReady(message);
        case _Constants.ExperimentConfigurationNotFound:
          throw ExperimentNotFound(message);
        case _Constants.FeatureFlagConfigurationNotFound:
          throw FeatureNotFound(message);
        case _Constants.FeatureFlagVariableNotFound:
          throw VariableNotFound(message: message);
        case _Constants.NotTargeted:
          throw NotTargeted(message);
        case _Constants.NotAllocated:
          throw NotAllocated(message);
        case _Constants.VisitorCodeMaxLength:
        case _Constants.VisitorCodeEmpty:
          throw VisitorCodeNotValid(message);
        case _Constants.ClientConfiguration:
          throw CredentialsNotFound(message);
        case _Constants.VariationNotFound:
          throw VariationNotFound(message);
        default:
          throw Exception(exception);
      }
    }
  }
}

class _Constants {
  static const ExperimentConfigurationNotFound = 'ExperimentConfigurationNotFound',
  VariationNotFound = 'VariationNotFound',
  FeatureFlagConfigurationNotFound = 'FeatureFlagConfigurationNotFound',
  FeatureFlagVariableNotFound = 'FeatureFlagVariableNotFound',
  NotTargeted = 'NotTargeted',
  NotAllocated = 'NotAllocated',
  VisitorCodeMaxLength = 'VisitorCodeMaxLength',
  VisitorCodeEmpty = 'VisitorCodeEmpty',
  ClientConfiguration = 'ClientConfiguration',
  Initialization = 'Initialization';
}

/// Helper class KameleoonUtilsJs to make interaction with JS sdk (web platform)
@JS("kameleoon.KameleoonUtils")
class _KameleoonUtilsJs {
  external static Future<String> getVisitorCode(String domain, String? defaultVisitorCode);
}

/// Helper class KameleoonClientJs to make interaction with JS sdk (web platform)
@JS("kameleoon.KameleoonClient")
class _KameleoonClientJs {
  external _KameleoonClientJs(String siteCode, _SDKConfigurationTypeJs configuration);
  external initialize();
  external runWhenReady(Function readyCallback, Function failCallback,
      [int timeout]);
  external Future<void> trackConversion(_TrackConversionParamsType object);
  external Future<void> addData(String visitorCode, Iterable<_KameleoonDataTypeJs> args);
  external Future<dynamic> obtainFeatureVariable(var featureKeyOrId, String variableKey);
  external Future<dynamic> getFeatureFlagVariable(_GetFeatureFlagVariableParamsType object);
  external Future<String> getFeatureFlagVariationKey(String visitorCode, String featureKey);
  external Future<dynamic> getExperimentVariationData(int variationId);
  external Future<void> flushData(String visitorCode);
  external Future<bool> isFeatureFlagActive(String visitorCode, String featureKey);
  external Future<int> triggerExperiment(String visitorCode, int experimentID);
  external Future<dynamic> getRemoteData(String key);
  external void onConfigurationUpdate(Function func);
  external List<_ExperimentType> getExperiments();
  external List<_ExperimentType> getVisitorExperiments(String visitorCode, bool? isAllocated);
  external List<_FeatureFlagType> getFeatureFlags();
  external List<_FeatureFlagType> getVisitorFeatureFlags(String visitorCode);

}

@JS("kameleoon.KameleoonDataType")
class _KameleoonDataTypeJs {
  external _KameleoonDataTypeJs();
}

@JS("kameleoon.CustomData")
class _CustomDataJs extends _KameleoonDataTypeJs {
  external Iterable<String> value;
  external _CustomDataJs(int id, Iterable<String> value);
}

@JS("kameleoon.Browser")
class _BrowserJs extends _KameleoonDataTypeJs {
  external _BrowserJs(int id, double? version);
}

@JS("kameleoon.Device")
class _DeviceJs extends _KameleoonDataTypeJs {
  external _DeviceJs(String device);
}

@JS("kameleoon.PageView")
class _PageViewJs extends _KameleoonDataTypeJs {
  external _PageViewJs(_PageViewParametersType object);
}

@JS("kameleoon.Conversion")
class _ConversionJs extends _KameleoonDataTypeJs {
  external _ConversionJs(_ConversionParametersType object);
}

@JS()
@anonymous
class _SDKConfigurationTypeJs {
  external factory _SDKConfigurationTypeJs({int? updateInterval, String? environment});
}

@JS()
@anonymous
class _GetFeatureFlagVariableParamsType {
  external factory _GetFeatureFlagVariableParamsType({String visitorCode, String featureKey, String variableKey});
}

@JS()
@anonymous
class _TrackConversionParamsType {
  external factory _TrackConversionParamsType({String visitorCode, int goalId, double revenue});
}

@JS()
@anonymous
class _ConversionParametersType {
  external factory _ConversionParametersType({int goalId, double revenue, bool negative});
}

@JS()
@anonymous
class _PageViewParametersType {
  external factory _PageViewParametersType({String urlAddress, String title, Iterable referrers});
}

@JS()
@anonymous
class _ExperimentType {
  external int id;
  external factory _ExperimentType();
}

@JS()
@anonymous
class _FeatureFlagType {
  external String key;
  external factory _FeatureFlagType();
}


Map _jsToDart(jsObject) {
  return jsonDecode(stringify(jsObject));
}

@JS('JSON.stringify')
external String stringify(Object obj);
